=== modified file 'avatarproviders.py'
--- avatarproviders.py	2011-04-06 14:50:40 +0000
+++ avatarproviders.py	2011-09-03 21:18:21 +0000
@@ -55,12 +55,18 @@
         self.__queue = Queue.Queue()
 
         self.__provider_method = provider_method
-        self.__end_thread = False
 
     def stop(self):
         """ Stop this worker """
-        self.__end_thread = True
         self.__stop.set()
+        while self.__queue.qsize() > 0:
+            self.__queue.get_nowait()
+            self.__queue.task_done()
+        self.__queue.join()
+
+    @property
+    def is_running(self):
+        return not self.__stop.is_set()
 
     def set_callback_method(self, method):
         """ Fire the given callback method when treatment is finished """
@@ -71,20 +77,27 @@
 
         This id_field is for example with Gravatar the email address.
         """
-        self.__queue.put(id_field)
+        if self.is_running:
+            self.__queue.put(id_field)
+            if not self.is_alive():
+                self.start()
 
     def run(self):
         """Worker core code. """
-        while not self.__end_thread:
-            id_field = self.__queue.get()
-            # Call provider method to get fields to pass in the request
-            url = self.__provider_method(id_field)
-            # Execute the request
-            response = urllib2.urlopen(url)
-            # Fire the callback method
-            if not self.__callback_method is None:
-                self.__callback_method(response, id_field)
-            self.__queue.task_done()
+        while self.is_running:
+            try:
+                id_field = self.__queue.get_nowait()
+                # Call provider method to get fields to pass in the request
+                url = self.__provider_method(id_field)
+                # Execute the request
+                response = urllib2.urlopen(url)
+                # Fire the callback method
+                if not self.__callback_method is None:
+                    self.__callback_method(response, id_field)
+                self.__queue.task_done()
+            except Queue.Empty:
+                # There is no more work to do.
+                pass
 
 
 class AvatarProviderGravatar(AvatarProvider):

=== modified file 'avatarsbox.py'
--- avatarsbox.py	2011-04-10 18:44:39 +0000
+++ avatarsbox.py	2011-09-03 21:18:21 +0000
@@ -38,7 +38,7 @@
 
     def __eq__(self, other):
         return (self.apparent_username == other.apparent_username and
-                self.name == other.name and
+                self.username == other.username and
                 self.email == other.email)
 
     def show_spinner(self):
@@ -184,7 +184,12 @@
         # This callback method should be fired by all workers when a request
         # is done.
         self.__worker.set_callback_method(self._update_avatar_from_response)
-        self.__worker.start()
+        self.connect('destroy', self.on_destroy)
+
+    def on_destroy(self, widget):
+        self.__worker.stop()
+        if self.__worker.is_alive():
+            self.__worker.join()
 
     def add(self, username, role):
         """Add the given username in the role box and add in the worker queue.

=== modified file 'tests/test_annotate_config.py'
--- tests/test_annotate_config.py	2009-05-07 11:48:03 +0000
+++ tests/test_annotate_config.py	2011-09-03 21:18:21 +0000
@@ -25,11 +25,18 @@
     gannotate,
     )
 
+
 class TestConfig(tests.TestCaseInTempDir):
 
     def setUp(self):
+        # Create an instance before the env is changed so that
+        # icon lookups work.
+        self.window = gannotate.GAnnotateWindow()
         super(TestConfig, self).setUp()
-        self.window = gannotate.GAnnotateWindow()
+
+    def tearDown(self):
+        self.window.destroy()
+        super(TestConfig, self).tearDown()
 
     def test_create_initial_config(self):
         """We can create a config even without a prior conf file"""

=== modified file 'tests/test_revisionview.py'
--- tests/test_revisionview.py	2011-07-28 15:34:17 +0000
+++ tests/test_revisionview.py	2011-09-03 21:18:21 +0000
@@ -16,6 +16,8 @@
 
 """Test the RevisionView functionality."""
 
+import os
+
 from bzrlib import (
     tests,
     )
@@ -27,8 +29,17 @@
 from bzrlib.plugins.gtk import revisionview
 
 
+def fake_icon_path(*args):
+    return os.path.join(
+        os.path.dirname(__file__),  '..', 'icons', *args)
+
+
 class TestPendingRevisions(tests.TestCaseWithMemoryTransport):
 
+    def setUp(self):
+        super(TestPendingRevisions, self).setUp()
+        self.overrideAttr(revisionview, 'icon_path', new=fake_icon_path)
+
     def assertBufferText(self, text, buffer):
         """Check the text stored in the buffer."""
         self.assertEqual(text, buffer.get_text(buffer.get_start_iter(),
@@ -41,6 +52,7 @@
         b = builder.get_branch()
 
         rv = revisionview.RevisionView(b)
+        self.addCleanup(rv.destroy)
         rev = b.repository.get_revision('A')
         rv.set_revision(rev)
         self.assertEqual(rev.committer, rv.committer.get_text())
@@ -61,6 +73,7 @@
         b = tree.branch
 
         rv = revisionview.RevisionView(b)
+        self.addCleanup(rv.destroy)
         rev = b.repository.get_revision('A')
         rv.set_revision(rev)
 
@@ -81,6 +94,7 @@
         b = tree.branch
 
         rv = revisionview.RevisionView(b)
+        self.addCleanup(rv.destroy)
         rev = b.repository.get_revision('A')
         rv.set_revision(rev)
 

